#if !defined(CONST_H)
#define CONST_H

/* This one is IMPORTANT. It's the number of output parameters. If you're
going to change the output, then you'll have to change this, otherwise very
bad things will happen involving unallocated array space. */
#define OUTPUT_COUNT 30
#define SOUNDING_LEVELS 50

/* Maximum string length. This is applied to *all* the strings, e.g. output
parameter names etc. */
#define MAX_STRING_LENGTH 80 

/* Header file containing an assortment of physical and other constants,
some of which aren't. Constant, that is. Not really. */

#define KTS_TO_METRES 1.944          /* Knots to metres per second */
#define FT_TO_METRES 3.281           /* Feet to metres */
#define DEGREES_TO_RADIANS 0.0174533 /* Degrees to radians */
#define C_TO_K 273.15     /* Celcius to kelvin */
#define RADIANS_TO_DEGREES 57.29578  /* Radians to degrees. Why not just use degrees to radians defined above?*/
#define ROT_RATE_EARTH 7.27E-5       /* Rotation rate of the earth. Units? Radians per second */
#define DENS 1.1
#define KARMAN 0.4
#define KONST 0.74
#define RROE 1.83
#define RHO_W 1000                   /* Density of water in kg/m^3 */
#define FT 1.0
#define RHOW 1000

/* Some other constants that definitely aren't physical, but we
may as well keep them here as anywhere else. */
#define DPROF_LEVELS 52
#define DPROF_THICKLEVEL 50
#define DPROF_VARCOUNT 4
#define DPROF_ROWCOUNT 0

/* Constants for the water budget equation, used by water() */
#define W_CONST1 1
#define W_CONST2 0.5
#define W_D1P 0.1
#define W_DINT 0.5
#define W_D2P 0.5
#define W_OMG 24

/* Constants use by bri */
#define A 8.3e-4
#define DT 90
#define Z1 1.0
#define B 0.2
#define ANEW 1.0
#define BNEW 0.667
#define CNEW 5.0
#define DNEW 0.35

/* snding */
#define GRAV 9.78

/* vel */
#define CP 1004.832
#define PI 3.141592654

/* water */

#define CONST1 1
#define CONST2 0.5
#define DIP 0.1
#define DINT 0.5
#define D2P 0.5
#define OMG 24

#endif

