
#include <math.h>
#include "daykm.h"
#include "const.h"

/* Function to computer eddy diffusivities as a function of height,
friction velocity and Monin Obukhov length for all the relevant levels
in the mixing layer at each time step during the day; using the method
of O'Brien. Used in Momday. */

float _daykm(float *km, float *zi, float *zk, float hgt, float za,
		float ustar, float mol, int ntrp,
		float deltaz)
{
	float ktop, kma, kmapri, thick;
	/* Array declared of size 51 so we can use fortran-style
	unit offset indexing. */
	float kmw[51];
	int m,i;

	thick=hgt-za;

	/* only do this if there is a mixing layer. */
	if(thick!=0)
	{
		/* Define or calc eddy diff's (K's) at the top of the
		surface layer using the standard flux profile law.
		Calculate the derivative of kma and define k at the top
		of the mixing layer to be zero. */
		kma=KARMAN*ustar*za*pow((float)(1.0-(15*za/mol)),0.25);
		km[1]=kma;
		kmw[1]=kma;
		kmapri=kma*(1/za-(15.0/mol)/(1.0-15.0*za/mol));
		ktop=0;

		/* Calc. heights between the 250 metre intervals (zi
		system)	as passed from spline, and call zk system
		(interval 50,175,425 etc.) */
		m=ntrp+1;
		for(i=2;i<=m;i++)
		{
			zk[i]=zi[i]-0.5*deltaz;
		}
		zk[1]=zi[1];

		/* Calc. the eddy diffusivities for momentum and water
		up to the top of the mixing layer using the O'Brien
		function. If ntrp goes abovehgt, fill with zeros. */
		for(i=2;i<=ntrp;i++)
		{
			if(zk[i]<hgt)
			{
				km[i]=ktop+(pow((float)(zk[i]-hgt),2)/(thick*thick))*
					(kma-ktop+(zk[i]-za)*(kmapri+2*(kma-ktop)/thick));

				kmw[i]=ktop+(pow((float)(zi[i]-hgt),2)/(thick*thick))*
					(kma-ktop+(zi[i]-za)*(kmapri+2*(kma-ktop)/thick));
			}
			else
			{
				km[i]=0;
				kmw[i]=0;
			}
		}

		/* Smooth k's as a weighted average */
		for(i=2;i<=ntrp;i++)
		{
			km[i]=(kmw[i]+km[i]+kmw[i-1])/3.0;
		}
	} /* End of thick!=0 conditional */
   return(thick);
}
