
#include <stdio.h>
#include <stdlib.h>
#include "psubams.h"
#include "const.h"
#include "dectim.h"

int main(int argc, char **argv)
{

	/* Declare the input parameters */
   int iyr=0, imo=0, iday=0, cld_fract=0, mintemp=0, maxtemp=0, index_soils=0,
   	i=0, index_veggies=0, nobs_ptq=0, nobs_wind=0, num_soils=0, num_of_veggies=0;

   float tz=0,  xlat=0,  xlong=0, strtim=0,  timend=0,  outtt=0, slope=0, aspect=0, f=0, fsub=0,
   	wmax=0, btemp=0, tp=0, ti_a=0, ti_b=0, albg=0, epsi=0, omega=0, zo=0, obst_hgt=0, frveg=0,
      xlai=0, epsf=0, albf=0, volrel=0, rmin=0,  rcut=0,  wilt=0,  vegheight=0,  width=0, ci=0, co=0,
      coz_sfc=0, coz_air=0, rks=0, cosbyb=0, thmax=0, psis=0, beta=0, b1=0, b2=0, psice=0, sc=0, zp=0,
      frhgt=0, frzp=0, rkocap=0, rccap=0, rzcap=0, volini=0, zstini=0, station_height=0, ugs=0,
      vgs=0;

   char albedo_gflag, dual_ti, cloud_flag, albedo_fflag, stmtype, steady;

   float ps[51], ts[51], dep[51], dd0[51], ff0[51], zh[51];

	char soiltype[16], planttype[31];

	/* Declare an assortment of variables */
   FILE *i_model, *soils, *veglut;
   int file_error;

   int start_t, end_t, j, no_rows, intv;

   /* Arrays for return of data */
   float **output_data, ***sounding_data;
   char **output_names;

   /* Zero arrays */
   for(i=0;i<51;i++)
   {
   	ps[i]=0;
      ts[i]=0;
      dep[i]=0;
      dd0[i]=0;
      ff0[i]=0;
      zh[i]=0;
   }
	/* Open input file. */
   i_model=fopen("i_model.dat","r");
   if(i_model==NULL)
   {
   	fprintf(stderr,"Failure to open i_model.dat for reading\n");
      exit(EXIT_FAILURE);
   }

   /* Read all the input parameters, in no particular order. */
   file_error=fscanf(i_model,"%d %d %d %f %f %f %f %f %f %f %f",&iyr, &imo, &iday,
   		&tz, &xlat, &xlong, &strtim, &timend, &outtt, &slope, &aspect);
   if(file_error!=11)
   {
   	fprintf(stderr,"Failure to read line 1 of input file. (%d/11)\n",file_error);
      exit(EXIT_FAILURE);
   }

   file_error=fscanf(i_model,"%f %f %f %f %f %c %f %f %c %f %f %d", &f,
   		&fsub, &wmax, &btemp, &tp, &dual_ti, &ti_a, &ti_b, &albedo_gflag,
         &albg, &epsi, &index_soils);
	if(file_error!=12)
   {
   	fprintf(stderr,"Failure to read line 2 of input file. (%d/11)\n",file_error);
      exit(EXIT_FAILURE);
   }

	file_error=fscanf(i_model,"%f %f %f %c %d", &omega, &zo, &obst_hgt, &cloud_flag, &cld_fract);
   if(file_error!=5)
   {
   	fprintf(stderr,"Failure to read line 3 of input file. (%d/11)\n",file_error);
      exit(EXIT_FAILURE);
   }

	file_error=fscanf(i_model,"%f %f %f %c %f %c %d %f %f %f %f %f %f %c %f %f %f %f",
   		&frveg, &xlai, &epsf, &albedo_fflag, &albf, &stmtype, &index_veggies,
         &volrel, &rmin, &rcut, &wilt, &vegheight, &width, &steady, &ci, &co,
         &coz_sfc, &coz_air);

   if(file_error!=18)
   {
   	fprintf(stderr,"Failure to read line 4 of input file. (%d/18\n",file_error);
      exit(EXIT_FAILURE);
   }

   /* The input file should still be open */
   file_error=fscanf(i_model,"%d %d %f %f %f", &nobs_ptq, &nobs_wind, &station_height, &ugs, &vgs);
   if(file_error!=5)
   {
   	fprintf(stderr,"Error reading header line of sounding file\n");
      exit(EXIT_FAILURE);
   }

   for(i=1;i<=nobs_ptq;i++)
   {
   	file_error=fscanf(i_model,"%f %f %f",&ps[i], &ts[i], &dep[i]);
      if(file_error!=3)
      {
      	fprintf(stderr,"Error reading line %d of p/t/q obs\n",i);
         exit(EXIT_FAILURE);
      }
   }
   for(i=1;i<=nobs_wind;i++)
   {
   	file_error=fscanf(i_model,"%f %f %f",&dd0[i], &ff0[i], &zh[i]);
      if(file_error!=3)
      {
      	fprintf(stderr,"Error reading line %d of wind obs\n",i);
         exit(EXIT_FAILURE);
      }
   }
	/* Close the input files */
   fclose(i_model);

   /* Read extra paramters under certain conditions. Note:
   1. These paramters will not necessarily be used.
   2. This section will overwrite the values of rmin and rcut defined adove. */
   if((frveg>0)&&(stmtype=='L'))
   {
   	/* Open and read the soil database */
   	soils=fopen("soils.dat","r");
      if(soils==NULL)
      {
      	fprintf(stderr,"Error opening soil database\n");
         exit(EXIT_FAILURE);
      }

      file_error=fscanf(soils,"%d",&num_soils); /* Read the number of soils in the list. */
      if(file_error!=1)
      {
      	fprintf(stderr,"Error reading number of soils from database\n");
         exit(EXIT_FAILURE);
      }

      for(i=1;i<=index_soils;i++)
      {
      	file_error=fscanf(soils,"%s %f %f %f %f",soiltype, &rks, &cosbyb, &thmax, &psis);
      	if(file_error!=5)
         {
         	fprintf(stderr,"Failure to read line %d of soils.dat (%d/5)\n",i,file_error);
            exit(EXIT_FAILURE);
         }
      }
      /* Close the soil data file */
      fclose(soils);

      /* Open and read the plant database */
      veglut=fopen("veglut.dat","r");
      if(veglut==NULL)
      {
      	fprintf(stderr,"Error opening plant databse\n");
         exit(EXIT_FAILURE);
      }
      fscanf(veglut,"%d",&num_of_veggies);
      for(i=1;i<=index_veggies;i++)
      {
      	file_error=fscanf(veglut,"%s %f %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
         	planttype, &rmin, &mintemp, &maxtemp, &beta, &b1, &b2, &psice,
            &sc, &rcut, &zp, &frhgt, &frzp, &rkocap, &rccap, &rzcap, &volini,
            &zstini);
         if(file_error!=18)
         {
         	fprintf(stderr,"Failure to read line %d of veglut.dat (%d/18)\n", i, file_error);
            exit(EXIT_FAILURE);
         }
      }
      /* Close the vegetation data file */
      fclose(veglut);

   } /* End of if((frveg>0)&&(stmtype=='L')) */

   /* Figure out how big the arrays for data need to be, and allocate the
   necessary space. This requires duplicating some of the code that's in
   _psubams, to figure out the number of rows of data, but I prefer to
   allocate and free memory in the same part of a program if at all possible. */

	/* Get the start and end times */
   start_t=_dectim(strtim);
   end_t=_dectim(timend);
   intv=_dectim(outtt);
   no_rows=((end_t-start_t)/intv)+1;

   /* Allocate columns first */
   /* See const.h for definition of OUTPUT_COUNT. Note that it would have to
   be changed (and the program recompiled) if any output parameters are
   added or removed. */

   /* To get to an individual value; output_data[param_index][time index] */
   output_data=calloc(OUTPUT_COUNT,sizeof(float*));

   /* Now allocate all the rows, one for each time step */
   for(i=0;i<OUTPUT_COUNT;i++)
   {
   	output_data[i]=calloc(no_rows,sizeof(float));
   }


	/* Do the same thing for the sounding array, although this is going to be
   uglier, because it's in 3 dimensions; timestep, height, parameter.
   At least we know how many parameters there are going to be; always
   height, pressure, temperature, humidity, u component of wind, v component
   of wind, in that order. Probably. In other words, there are seven. */

   /* This is set up so an individual value can be obtained using:
   sounding_data[param(i.e. 0=height, etc.)][time index][height index] */

   sounding_data=calloc(7,sizeof(float**));

   /* Allocate all the rows, one for each time step. */
   for(i=0;i<7;i++)
   {
   	sounding_data[i]=calloc(no_rows,sizeof(float*));

      /* And now allocate the rows within the rows, for the number of levels. */
      /* The number of levels is defined in const.h as well. */
      for(j=0;j<no_rows;j++)
      {
      	sounding_data[i][j]=calloc(SOUNDING_LEVELS,sizeof(float));
      }
   }

   /* Allocate space for the names of the output paramters. These can't be
   any longer than the maximum size defined by MAX_STRING_LENGTH (in const.h)
   If/when adding names, one should beware of this. */
	output_names=calloc(OUTPUT_COUNT,sizeof(char *));
   for(i=0;i<OUTPUT_COUNT;i++)
   {
   	output_names[i]=calloc(MAX_STRING_LENGTH,sizeof(char));
   }

   /* Call the big lump of the model. */
   _psubams(iyr, imo, iday, tz, xlat, xlong, strtim, timend, outtt, slope,
   	aspect, f, fsub, wmax, btemp, tp, dual_ti, ti_a, ti_b, albedo_gflag,
      albg, epsi, omega, zo, obst_hgt, cloud_flag, cld_fract, frveg, xlai,
      epsf, albedo_fflag, albf, stmtype, volrel, rmin, rcut, wilt, vegheight,
      width, steady, ci, co, coz_sfc, coz_air, rks, cosbyb, thmax, psis,
      mintemp, maxtemp, beta, b1, b2, psice, sc, zp, frhgt, frzp, rkocap,
      rccap, rzcap, volini, zstini,
      nobs_ptq, nobs_wind, station_height, ugs, vgs,
      ps, ts, dep, dd0, ff0, zh);

   /* Dellocate all the memory set aside for the output stuff. I could be lazy
   and not do this, but whatever. */
	for(i=0;i<OUTPUT_COUNT;i++)
   {
   	free(output_data[i]);
   }
   free(output_data);

   for(i=0;i<7;i++)
   {
   	for(j=0;j<no_rows;j++)
      {
      	free(sounding_data[i][j]);
      }
      free(sounding_data[i]);
   }
   free(sounding_data);

   /* Done. Leave. */
   return(0);
}
